define(['exports', 'react', 'lodash', 'mobx', 'jinrishici'], function (exports, react, lodash, mobx, jinrishici) { 'use strict';

  function _interopNamespace(e) {
    if (e && e.__esModule) return e;
    var n = Object.create(null);
    if (e) {
      Object.keys(e).forEach(function (k) {
        if (k !== 'default') {
          var d = Object.getOwnPropertyDescriptor(e, k);
          Object.defineProperty(n, k, d.get ? d : {
            enumerable: true,
            get: function () {
              return e[k];
            }
          });
        }
      });
    }
    n['default'] = e;
    return Object.freeze(n);
  }

  var jinrishici__namespace = /*#__PURE__*/_interopNamespace(jinrishici);

  var useMount = function (fn) {
      react.useEffect(function () {
          fn();
      }, []);
  };

  var useUnmount = function (fn) {
      var fnRef = react.useRef(fn);
      // 如果 fn 的依赖更新了，就要更新卸载要执行的方法
      fnRef.current = fn;
      react.useEffect(function () {
          return function () {
              fnRef.current();
          };
      }, []);
  };

  var useEffectOnce = function (mountFn, unMountFn) {
      if (lodash.isFunction(mountFn)) {
          useMount(mountFn);
      }
      if (lodash.isFunction(unMountFn)) {
          useUnmount(unMountFn);
      }
  };

  var useUpdate = function () {
      var _a = react.useState({}), setState = _a[1];
      return react.useCallback(function () { return setState({}); }, []);
  };

  var useUpdateEffect = function (effect, deps) {
      var isUpdate = react.useRef(false);
      react.useEffect(function () {
          if (!isUpdate.current) {
              isUpdate.current = true;
          }
          else {
              effect();
          }
      }, deps);
  };

  /*! *****************************************************************************
  Copyright (c) Microsoft Corporation.

  Permission to use, copy, modify, and/or distribute this software for any
  purpose with or without fee is hereby granted.

  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
  REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
  AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
  INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
  LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
  PERFORMANCE OF THIS SOFTWARE.
  ***************************************************************************** */

  function __spreadArray(to, from, pack) {
      if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
          if (ar || !(i in from)) {
              if (!ar) ar = Array.prototype.slice.call(from, 0, i);
              ar[i] = from[i];
          }
      }
      return to.concat(ar || Array.prototype.slice.call(from));
  }

  function resolveState(nextState, currentState) {
      if (lodash.isFunction(nextState)) {
          return nextState.length
              ? nextState(currentState)
              : nextState();
      }
      return nextState;
  }

  var useArray = function (initialValue) {
      if (initialValue === void 0) { initialValue = []; }
      var array = react.useRef(resolveState(initialValue));
      var update = useUpdate();
      var actions = react.useMemo(function () {
          var action = {
              set: function (newArray) {
                  array.current = resolveState(newArray, array.current);
                  update();
              },
              reset: function () {
                  action.set(resolveState(initialValue).slice());
              },
              push: function () {
                  var item = [];
                  for (var _i = 0; _i < arguments.length; _i++) {
                      item[_i] = arguments[_i];
                  }
                  if (item.length > 0) {
                      action.set(function (cur) { return cur.concat(item); });
                  }
              },
              updateAt: function (index, item) {
                  action.set(function (cur) {
                      var arr = __spreadArray([], cur, true);
                      arr[index] = item;
                      return arr;
                  });
              },
              insertAt: function (index) {
                  var item = [];
                  for (var _i = 1; _i < arguments.length; _i++) {
                      item[_i - 1] = arguments[_i];
                  }
                  action.set(function (cur) {
                      var arr = __spreadArray([], cur, true);
                      arr.splice.apply(arr, __spreadArray([index, 0], item, false));
                      return arr;
                  });
              },
              removeAt: function (index) {
                  action.set(function (cur) {
                      var arr = __spreadArray([], cur, true);
                      arr.splice(index, 1);
                      return arr;
                  });
              },
              clear: function () {
                  action.set([]);
              },
          };
          return action;
      }, []);
      return [array.current, actions];
  };

  var cachedCodes = new Map();
  var useCode = function (code, queryCode, refresh) {
      var _a = react.useState([]), data = _a[0], setData = _a[1];
      react.useEffect(function () {
          var cacheData = cachedCodes.get(code);
          if (!code)
              return;
          if (!cacheData || refresh) {
              queryCode(code).then(function (data) {
                  setData(data);
                  cachedCodes.set(code, data);
              });
              return;
          }
          if (cacheData) {
              setData(cacheData);
          }
      }, [code]);
      return data;
  };

  var cachedDataSet = new Map();
  var useDataSet = function (dataSetFactory, cacheKey) {
      return react.useMemo(function () {
          if (cacheKey) {
              var cacheDS = cachedDataSet.get(cacheKey);
              if (!cacheDS) {
                  cacheDS = dataSetFactory();
                  cachedDataSet.set(cacheKey, cacheDS);
              }
              return cacheDS;
          }
          return dataSetFactory();
      }, []);
  };

  var useDataSetIsSelected = function (dataSet) {
      var isSelectedFun = function () {
          return dataSet.selected.length > 0;
      };
      var _a = react.useState(isSelectedFun()), isSelected = _a[0], setIsSelected = _a[1];
      react.useEffect(function () {
          var disposer = mobx.autorun(function () {
              setIsSelected(isSelectedFun());
          });
          return disposer;
      }, []);
      return isSelected;
  };

  var useDataSetEvents = function (dataSet, eventNames, handler, options) {
      var handlerRef = react.useRef();
      handlerRef.current = handler;
      var eventListener = function (event) {
          return handlerRef.current && handlerRef.current(event);
      };
      var handleEventListener = react.useCallback(function (isAdd) {
          var listener = function (eventName) {
              if (isAdd) {
                  dataSet.addEventListener(eventName, eventListener, {
                      capture: options === null || options === void 0 ? void 0 : options.capture,
                      once: options === null || options === void 0 ? void 0 : options.once,
                      passive: options === null || options === void 0 ? void 0 : options.passive,
                  });
              }
              else {
                  dataSet.removeEventListener(eventName, eventListener, {
                      capture: options === null || options === void 0 ? void 0 : options.capture,
                  });
              }
          };
          if (lodash.isArray(eventNames)) {
              eventNames.forEach(function (eventName) {
                  listener(eventName);
              });
          }
          else if (lodash.isString(eventNames)) {
              listener(eventNames);
          }
      }, [dataSet, handler]);
      react.useEffect(function () {
          handleEventListener(true);
          return function () {
              handleEventListener(false);
          };
      }, [eventNames, options === null || options === void 0 ? void 0 : options.capture, options === null || options === void 0 ? void 0 : options.once, options === null || options === void 0 ? void 0 : options.passive]);
  };

  var useTodayPoetry = function (intervalTime) {
      var _a = react.useState({
          content: '',
          matchTags: [],
          origin: {},
      }), poetryData = _a[0], setPoetryData = _a[1];
      var _b = react.useState(false), loading = _b[0], setLoading = _b[1];
      react.useEffect(function () {
          var timer;
          init();
          if (lodash.isNumber(intervalTime) && intervalTime > 0) {
              timer = setInterval(function () {
                  init();
              }, intervalTime);
          }
          return function () {
              clearInterval(timer);
          };
      }, [intervalTime]);
      var init = react.useCallback(function () {
          setLoading(true);
          jinrishici__namespace.load(function (result) {
              setLoading(false);
              if (result.status === 'success') {
                  setPoetryData(result.data);
              }
          }, function (errData) {
              setLoading(false);
              console.error(errData);
          });
      }, []);
      return {
          content: poetryData.content,
          tags: poetryData.matchTags || [],
          origin: poetryData.origin,
          loading: loading,
          allData: poetryData,
          update: init,
      };
  };

  exports.useArray = useArray;
  exports.useCode = useCode;
  exports.useDataSet = useDataSet;
  exports.useDataSetEvents = useDataSetEvents;
  exports.useDataSetIsSelected = useDataSetIsSelected;
  exports.useEffectOnce = useEffectOnce;
  exports.useMount = useMount;
  exports.useTodayPoetry = useTodayPoetry;
  exports.useUnmount = useUnmount;
  exports.useUpdate = useUpdate;
  exports.useUpdateEffect = useUpdateEffect;

  Object.defineProperty(exports, '__esModule', { value: true });

});
